<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
   <xsl:output method="html" indent="yes" encoding="US-ASCII"/>
   
   <xsl:template match="module">
      <html>
         <head>
            <title>
               .: Automatic Documentation :. 
               MODULE - <xsl:value-of select="@name" />
            </title>
            <style type="text/css">
               @import "doc.css";
            </style>
         </head>
         <body>
            
            <xsl:if test="file">
               <div class="file">
                  <xsl:apply-templates select="file" />
                  <ul>
                     <xsl:apply-templates select="file/sr">	
                     <xsl:sort select="@name" />
                     </xsl:apply-templates>
                  </ul>
               </div>
            </xsl:if>

            <xsl:if test="child">
               <div class="child">
                  <xsl:apply-templates select="child" />
                  <ul>
                     <xsl:apply-templates select="child/sr">	
                     <xsl:sort select="@name" />
                     </xsl:apply-templates>
                  </ul>
               </div>
            </xsl:if>
            
            <xsl:if test="parent">
               <div class="parent">
                  <xsl:apply-templates select="parent" />
                  <ul>
                     <xsl:apply-templates select="parent/sr">
                        <xsl:sort select="@name" />
                     </xsl:apply-templates>
                  </ul>
               </div>
            </xsl:if>
            
            <div class="module">
               <h1>
                  <span class="version">Version : 
                  <xsl:choose>
                     <xsl:when test="@version">
                        <xsl:value-of select="@version" />
                     </xsl:when>
                     <xsl:otherwise>
                        1.0
                     </xsl:otherwise>
                  </xsl:choose>
                  </span>
                  File -- <xsl:value-of select="@name" /> --
               </h1>

               <xsl:apply-templates select="table" />
               
               <xsl:apply-templates select="code" />
               
            </div>

            <div class="navigator">
               Files:<a href="index.html">index.html</a>
            </div>
         </body>
      </html>
   </xsl:template>
   
   <xsl:template match="file">
      <h2><xsl:value-of select="@name" /></h2>
   </xsl:template>

   <xsl:template match="child">
      <h2><xsl:value-of select="@name" /></h2>
   </xsl:template>

   <xsl:template match="parent">
      <h2><xsl:value-of select="@name" /></h2>
   </xsl:template>

   <xsl:template match="sr">
      <li>
         <xsl:choose>
            <xsl:when test="@option='file'">
               <a>
                  <xsl:attribute name="href"><xsl:value-of select="@file" />.html</xsl:attribute>
                  <xsl:attribute name="title"><xsl:value-of select="@file" /></xsl:attribute>
                  <xsl:value-of select="@file" />
               </a>
            </xsl:when>
            <xsl:when test="@option='infile'">
               <a>
                  <xsl:attribute name="href">
                     <xsl:value-of select="@file" />.html#<xsl:value-of select="@name" />
                  </xsl:attribute>
                  <xsl:attribute name="title"><xsl:value-of select="@name" /></xsl:attribute>
                  <xsl:value-of select="@name" /> -- (<xsl:value-of select="@file" />)
               </a>
            </xsl:when>
            <xsl:otherwise>
               <a>
                  <xsl:attribute name="href">#<xsl:value-of select="@name" /></xsl:attribute>
                  <xsl:attribute name="title"><xsl:value-of select="@name" /></xsl:attribute>
                  <xsl:value-of select="@name" />
               </a>
               <xsl:if test="pr">
                  <ul>
                     <xsl:apply-templates select="pr">	
                        <xsl:sort select="@name" />
                     </xsl:apply-templates>
                  </ul>
               </xsl:if>
            </xsl:otherwise>
         </xsl:choose>
      </li>
   </xsl:template>

   <xsl:template match="pr">
      <li>
         <a>
            <xsl:attribute name="href">
               <xsl:value-of select="@file" />.html#<xsl:value-of select="@name" /></xsl:attribute>
            <xsl:attribute name="title"><xsl:value-of select="@name" /></xsl:attribute>
            <xsl:value-of select="@name" /> -- (<xsl:value-of select="@file" />)
         </a>
      </li>
   </xsl:template>

   <xsl:template match="name">
         <a>
            <xsl:attribute name="name"><xsl:value-of select="@name" /></xsl:attribute>
            <xsl:attribute name="title"><xsl:value-of select="@name" /></xsl:attribute>
            <xsl:value-of select="@name" />
         </a>
   </xsl:template>

   <xsl:template match="link">
      <a>
         <xsl:attribute name="href">
            <xsl:value-of select="@file" />.html#<xsl:value-of select="@name" />
         </xsl:attribute>
         <xsl:attribute name="title">
            <xsl:value-of select="@file" />.html#<xsl:value-of select="@name" />
         </xsl:attribute>
         <xsl:value-of select="./text()" />
      </a>
   </xsl:template>
   
   <xsl:template match="table">
      <xsl:choose>
         <xsl:when test="@option='table'">
            <h2><xsl:value-of select="@name" /></h2>
            <table>
               <xsl:apply-templates select="table" />
            </table>
         </xsl:when>
         <xsl:when test="@option='line'">
            <tr>
               <xsl:apply-templates select="table" />
            </tr>
         </xsl:when>
         <xsl:when test="@option='item'">
            <td>
               <xsl:apply-templates select="link"/>
            </td>
         </xsl:when>
         <xsl:otherwise>
            <xsl:value-of select="./text()" />
            <xsl:text> </xsl:text>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>
   
   <xsl:template match="code">
      <pre>
         <xsl:apply-templates/>
      </pre>
   </xsl:template>
  
   <xsl:template match="comment">
      <em>
         <xsl:apply-templates/>
      </em>
   </xsl:template>
   
   
</xsl:stylesheet>
