#!/usr/bin/env python
#------------------------------------------------------
#Conversion of the CPMD code into a documentation
#Date: 14/10/2003
#------------------------------------------------------

#Plan
#----
#Build the documentation of the code with hyperlinks.

#ToDo
#----
# Detect pre-processor command
# Add links for each variable belonged to an include file.
# Debug --html option

#Message
def usage(error=0):
    print """
docf90 [--help] [--update] source new
    where:
      source   is the directory that contains CPMD sources
      new      is the new directory
      --help   displays this message
      --html   build html files from xml files
      --update replaces only the missing files
"""
    sys.exit(error)


import getopt
import os
import cPickle
import re
import string
import sys
import time
#import tempfile

try:
    optlist, args = getopt.getopt(sys.argv[1:],'hu',["help","update","html"])
except getopt.error:
    print 'Error in arguments'
    usage(error=1)
    

update = 0
do_html = 0
for opt,a in optlist:
    if opt == '-h' or opt == '--help':
        usage(error=0)
    elif opt == '-u' or opt == '--update':
        update = 1
    elif opt == '--html':
        do_html = 1
    else:
        sys.stderr.write('%s: Unknown option\n')
        usage(error=1)

if len(args) != 2:
    sys.stderr.write('You have to specify the CPMD source directory\n')
    sys.stderr.write('and the new directory.\n')
    usage(error=1)
    
SRC = args[0]
NEW = args[1]


#Temporary file
#temp = tempfile.mktemp()

#Regular expressions used
re_assume_shape = re.compile('\(\*\)')
re_blankline = re.compile('^[ ]+&[ ,]*$')
re_call = re.compile('(?P<call>^[ ]+call[ ]*)(?P<name>\w+)',re.IGNORECASE)
re_common = re.compile('^[ ]+common',re.IGNORECASE)
re_declaration = re.compile('^[ ]+(character|complex|dimension|integer|logical|real)',re.IGNORECASE)
re_decl_pointer = re.compile("\([ ]*IP_(?P<name>\w+)[ ]*,[ ]*\\1[ ]*\)",re.IGNORECASE)
re_file = re.compile("'(?P<file>[a-zA-Z0-9_.]+)'",re.IGNORECASE)
re_get_addr = re.compile("=[ ]*get_addr[ ]*[(]",re.IGNORECASE)
re_get_addr_rep = re.compile("IP_(?P<name>\w+)[ ]*=[ ]*get_addr[ ]*[(]",re.IGNORECASE)
re_include = re.compile("^[ ]+include[ ]*'(?P<file>[a-zA-Z0-9_.]+)'",re.IGNORECASE)
re_ip = re.compile('ip_(?P<name>\w+)',re.IGNORECASE)
re_memory = re.compile('call[ ]*(freem|memory|scrget|scrptf|scrptr)',re.IGNORECASE)
re_parameter = re.compile('^[ ]+parameter',re.IGNORECASE)
re_pointer = re.compile('^[ ]+pointer',re.IGNORECASE)
re_point = re.compile('pointer',re.IGNORECASE)
re_ptr_allocated = re.compile('ptr_allocated',re.IGNORECASE)
re_ptr_associated = re.compile('ptr_associated',re.IGNORECASE)
re_save = re.compile('^[ ]+save',re.IGNORECASE)
re_star = re.compile('[*]')
re_sub = re.compile('(?P<type>^[ ]+(program|subroutine|((\w+[*]*\d*)*[ ]*function))[ ]*)(?P<name>\w+)',re.IGNORECASE)
re_string = re.compile("'[^']*'")
re_tokdim = re.compile('[^,]+',re.IGNORECASE)
re_token = re.compile('(\w+)',re.IGNORECASE)
#Three levels of parenthesis possible to detect
re_tokendim = re.compile(\
    '((\w+)[ ]*(\([^()]*([^()]*\([^()]*([^()]*\([^()]*\))*\))*[^()]*\)))')
re_tokenlist = re.compile('((\w+)([(]*))',re.IGNORECASE)
re_type = re.compile('^[ ]+(\w+[*]*\d*)',re.IGNORECASE)

#Replace &, < and >
re_amp = re.compile('&')
re_lt = re.compile('<')
re_gt = re.compile('>')

is_dimension = 'dimension'

#List of all files
AllFiles = os.listdir(SRC)

#List of include files
IncludeFiles = []
FortranFiles = []
OtherFiles = []
for file in AllFiles:
    if file[-1:] == "~" or file[:2] == ".#":
        continue
    elif file == "system.h":
        IncludeFiles.append(file)
    elif file[-3:] == "inc":
        IncludeFiles.append(file)
    elif file[-1:] == "F":
        FortranFiles.append(file)
    else:
        OtherFiles.append(file)

#Sort all files
IncludeFiles.sort()
FortranFiles.sort()

#The main dictionary
Dict = {}
#Contains for each include files the name of Fortran File which uses it.
Dict_Include = {}
#Contains for each subroutine the name of Fortran File.
Dict_Sub = {}
#Contains for each routine the parent routines
Dict_Parent = {}
#Contains for each routine the child routines
Dict_Child = {}

#True if the line is a continuation line
def is_continuation(line):
    return(len(line) > 0 and line[0] == ' ' and len(line) >= 5 and line[5] != ' ')

#Update: Load all the information inside the dictionaries
if update:
    print "Load information..."
    upd = open("docf90.dict","r")
    Dict = cPickle.load(upd)
    Dict_Include = cPickle.load(upd)
    Dict_Sub = cPickle.load(upd)
    Dict_Parent = cPickle.load(upd)
    Dict_Child = cPickle.load(upd)
    upd.close()

#Initialization to build html files
if do_html:
    try:
        import libxml2
    except:
        print "You need to have the libxml2 module."
        sys.exit(1)
    try:
        import libxslt
    except:
        print "You need to have the libxslt module."
        sys.exit(1)
    styledoc = libxml2.parseFile("doc2xhtml.xsl")
    style = libxslt.parseStylesheetDoc(styledoc)

#Catch the version number in the file header.F
re_version = re.compile("VERSION[ ]+(?P<version>[^']+)")
fd = open("%s/%s" % (SRC,"header.F"),'r')
Version = "None"
for line in fd:
    reg = re_version.search(line)
    if reg:
        Version = reg.groupdict()["version"]
        break
fd.close()
print "Version of CPMD: %s" % Version

#Catch the date
Date = time.strftime("%d/%m/%Y",time.gmtime())

#Process each include file.
print "Processing Include Files..."
ninclude = 0
for file in IncludeFiles:
    #Update: Work if the file does not exist
    if update and os.path.isfile("%s/%s.xml" % (NEW,file)):
        #The file does already exist
        continue
    ninclude = ninclude + 1
    Dict_Include[file] = []
    print "[%s]" % file,
    sys.stdout.flush()
    #First, we determine the list of pointers,
    old = open("%s/%s" % (SRC,file),'r')
    pointer = []
    is_pointer = 0
    while 1:
        line=old.readline()
        if not line:
            break
        if is_pointer and is_continuation(line):
            pass
        else:
            is_pointer = 0
        if re_pointer.search(line):
            is_pointer=1
        if is_pointer:
            #We determine the pointers
	    #We remove the comments
	    line = line[:string.find(line,'!')]
            pp = re_decl_pointer.findall(line)
            if pp:
                for x in pp:
                    pointer.append(string.upper(x))
    old.close()
    # Second, the type of each variable and
    # the rank of each variable.
    old = open("%s/%s" % (SRC,file),'r')
    is_type = 0
    Dict_type = {}
    Dict_dim = {}
    while 1:
        line=old.readline()
        if not line:
            break
        if line[0] != " ":
            #Line of comments
            continue
        if re_common.search(line):
            continue
        elif re_pointer.search(line):
            continue
        elif re_parameter.search(line):
            continue
        if not is_type and is_continuation(line):
            continue
        #OK it is a type line or a dimension line.
        if not is_continuation(line):
            res = re_type.match(line)
            if res:
                pos = res.regs[1][1]
            else:
                #We assume it is a comments line (with   !)
                continue
            is_type = res.group(1)
        else:
            #We begin the line after the character of continuation.
            pos = 6
        #Check if only the dimension is given
        if string.lower(is_type) == is_dimension:
            is_type = is_dimension
        #We remove the comments in the line
        c = string.find(line,'!')
        line = line[:c]
        #print "Type: %s" % is_type
        tokendim = re_tokendim.findall(line[pos:])
	for x in tokendim:
            #capitalize always the token
	    token=string.upper(x[1])
            if x[2]:
                dim='('+re_tokdim.sub('1',x[2][1:-1])+')'
            else:
                dim=''
            if is_type == is_dimension:
                Dict_dim[token] = dim
            else:
                Dict_type[token] = is_type
                Dict_dim[token] = dim
    #We put all the information inside the dictionary
    Dict[file] = [ pointer, Dict_type, Dict_dim ]
    old.close()
print "Done."

#We build information about pointers
Pointer = {}
for file in Dict:
    for pointer in Dict[file][0]:
        Pointer[pointer] = file

print "Processing Fortran Files..."
nfortran = 0
for file in FortranFiles:
    #Update: Work if the file does not exist
    if update and os.path.isfile("%s/%s.xml" % (NEW,file)):
        #The file does already exist
        continue
    nfortran = nfortran + 1
    print "[%s]" % file,
    sys.stdout.flush()
    #First we determine what variables have the pointer attribute.
    old = open("%s/%s" % (SRC,file),'r')
    pointer = []
    include_list = []
    Dict_type = {}
    Dict_dim = {}
    is_pointer = 0
    subroutine = ""
    list_sub = []
    while 1:
        line = old.readline()
        if not line:
            break
        reg = re_sub.search(line)
        if reg:
            #We store the name of the subroutine.
            subroutine = string.upper(reg.groupdict()["name"])
            list_sub.append(subroutine)
            Dict_Sub[subroutine] = file
        reg = re_include.search(line)
        if reg:
            incfile = reg.groups()[0]
            include_list.append(incfile)
            if Dict_Include.has_key(incfile):
                Dict_Include[incfile].append((file,subroutine))
            #We add all the pointer of this include file
            if Dict.has_key(incfile):
                for x in Dict[incfile][0]:
                    pointer.append(x)
                    if Dict[incfile][2].has_key(x):
                        Dict_dim[x] = Dict[incfile][2][x]
                    else:
                        Dict_dim[x] = '(1)'
        if is_pointer and is_continuation(line):
            pass
        else:
            is_pointer = 0
        if re_pointer.search(line):
            is_pointer = 1
        if is_pointer:
            #We add these pointers.
            line = line[:string.find(line,'!')]
            pp = re_decl_pointer.findall(line)
            if pp:
                for x in pp:
                    pointer.append(string.upper(x))
        #Build the parent information
        if re_call.search(line):
            reg = re_call.search(line)
            sub = string.upper(reg.groupdict()["name"])
            #Child routines
            if Dict_Child.has_key(subroutine):
                if not Dict_Child[subroutine].count(sub):
                     Dict_Child[subroutine].append(sub)
            else:
                Dict_Child[subroutine] = [ sub ]
            #Parent routines
            if subroutine == "":
                print "\nWarning: No subroutine name"
                print file,sub
                sys.exit(0)
            if Dict_Parent.has_key(sub):
                if not Dict_Parent[sub].count(subroutine):
                    Dict_Parent[sub].append(subroutine)
            else:
                Dict_Parent[sub] = [ subroutine ]
    #Print pointer
    old.close()
    #Add the information inside the dictionaries
    Dict[file] = [ pointer, {}, Dict_dim, list_sub ]
print "Done."

print "Build the documentation..."
for file in FortranFiles:
    #Update: Work if the file does not exist
    if update and os.path.isfile("%s/%s.xml" % (NEW,file)):
        #The file does already exist
        continue
    print "[%s]" % file,
    sys.stdout.flush()
    old = open("%s/%s" % (SRC,file),'r')
    new = open("%s/%s.xml" % (NEW,file),'w')
    #Header
    new.write('<?xml version="1.0" encoding="ISO-8859-15"?>\n')
    new.write('<?xml-stylesheet type="text/xsl" href="doc2xhtml.xsl"?>\n')
    new.write('<module name="%s" version="%s (%s)">\n' % (file,Version,Date))
    new.write('<code>\n')
    while 1:
        line=old.readline()
        if not line:
            break
        #Filters
        line = re_amp.sub('&amp;',line)
        line = re_lt.sub('&lt;',line)
        line = re_gt.sub('&gt;',line)
        if re_include.search(line):
            #We add a link
            line = re_file.sub('<link name="" file="\g<file>">'+"'\g<file>'</link>",line)
        elif re_sub.search(line):
            #We add a link
            sub = string.upper(re_sub.search(line).groupdict()["name"])
            line = re_sub.sub('\g<type><name name="%s">\g<name></name>' % sub,line)
        elif re_call.search(line):
            reg = re_call.search(line)
            sub = string.upper(reg.groupdict()["name"])
            if Dict_Sub.has_key(sub):
                line = re_call.sub('\g<call><link name="%s" file="%s">\g<name></link>' % (sub,Dict_Sub[sub]), line)
        if line[0] != " ":
            #Line of comments
            new.write('<comment>'+line[:-1]+'</comment>\n')
        else:
            new.write(line)
    #Close code structure.
    new.write('</code>\n')
    #Add child routines
    new.write('   <child name="Child Routines -- %s">\n' % file)
    for sub in Dict[file][3]:
        new.write('<sr name="%s">\n' % sub)
        if Dict_Child.has_key(sub):
            for child in Dict_Child[sub]:
                if Dict_Sub.has_key(child):
                    new.write('<pr name="%s" file="%s"/>\n' % (child,Dict_Sub[child]))
                else:
                    pass
                    #print "Warning: [%s] is not referenced" % parent
        new.write('</sr>')
    new.write('   </child>\n')
    #Add parent routines
    new.write('   <parent name="Parent Routines">\n')
    for sub in Dict[file][3]:
        new.write('<sr name="%s">\n' % sub)
        if Dict_Parent.has_key(sub):
            for parent in Dict_Parent[sub]:
                if Dict_Sub.has_key(parent):
                    new.write('<pr name="%s" file="%s"/>\n' % (parent,Dict_Sub[parent]))
                else:
                    pass
                    print "\nWarning: The parent [%s] is not referenced for the child [%s]." % (parent,sub)
                    print Dict_Parent[sub]
                    sys.exit(0)
        else:
            pass
            #print "Warning: %s has no parent" % sub
        new.write('</sr>')
    new.write('   </parent>\n')
    new.write('</module>\n')
    new.close()
    old.close()
    #Use xslt to build html files
    if do_html:
        doc = libxml2.parseFile("%s/%s.xml" % (NEW,file))
        result = style.applyStylesheet(doc, None)
        style.saveResultToFilename("%s/%s.html" % (NEW,file), result, 0)
print "Done."


#Build Documentation for include files
print "Build Documentation for include files..."
for incfile in IncludeFiles:
    if update and os.path.isfile("%s/%s.xml" % (NEW,incfile)):
        #The file does already exist
        continue
    print "[%s]" % incfile,
    sys.stdout.flush()
    #Build the documentation for this file.
    old = open("%s/%s" % (SRC,incfile),'r')
    new = open("%s/%s.xml" % (NEW,incfile),'w')
    #Header
    new.write('<?xml version="1.0" encoding="ISO-8859-15"?>\n')
    new.write('<?xml-stylesheet type="text/xsl" href="doc2xhtml.xsl"?>\n')
    new.write('<module name="%s" version="%s (%s)">\n' % (incfile,Version,Date))
    new.write('<code>\n')
    is_pointer = 0
    is_common = 0
    while 1:
        line=old.readline()
        if not line:
            break
        #Filters
        line = re_amp.sub('&amp;',line)
        line = re_lt.sub('&lt;',line)
        line = re_gt.sub('&gt;',line)
        if line[0] != " ":
            #Line of comments
            new.write('<comment>'+line[:-1]+'</comment>\n')
        else:
            new.write(line)
    #Close all data structure
    new.write('</code>\n')
    #We add information about routines using include files.
    new.write('   <child name="Parent Routines -- %s">\n' % incfile)
    for tt in Dict_Include[incfile]:
        new.write('<sr name="%s" file="%s" option="infile"/>\n' % (tt[1],tt[0]))
    new.write('   </child>\n')
    #We add information about variables.
    new.write('   <parent name="Pointer Variables">\n')
    for var in Dict[incfile][0]:
        new.write('<sr name="%s"/>\n' % var)
    new.write('   </parent>\n')
    new.write('</module>\n')
    old.close()
    new.close()
    #Use xslt to build html files
    if do_html:
        doc = libxml2.parseFile("%s/%s.xml" % (NEW,incfile))
        result = style.applyStylesheet(doc, None)
        style.saveResultToFilename("%s/%s.html" % (NEW,incfile), result, 0)
print "Done."


#Then we copy the other files
print "Copying other Files..."
for file in OtherFiles:
    if os.path.isfile("%s/%s" % (SRC,file)) :
        print "[%s]" % file,
        old = open("%s/%s" % (SRC,file),'r')
        new = open("%s/%s" % (NEW,file),'w')
        new.write(old.read())
        old.close()
        new.close()
print "Done."

#Save all the information
print "Save all the information..."
upd = open('docf90.dict','w')
cPickle.dump(Dict,upd)
cPickle.dump(Dict_Include,upd)
cPickle.dump(Dict_Sub,upd)
cPickle.dump(Dict_Parent,upd)
cPickle.dump(Dict_Child,upd)
upd.close()
print "Done."

#Build the index file.
print "Build the index file:"
new = open("%s/index.xml" % NEW,'w')
#Header
new.write('<?xml version="1.0" encoding="ISO-8859-15"?>\n')
new.write('<?xml-stylesheet type="text/xsl" href="doc2xhtml.xsl"?>\n')
new.write('<module name="Index of CPMD" version="%s (%s)">\n' % (Version,Date))
#Fortran Files
new.write('<!-- Fortran files -->\n')
new.write('<child name="Fortran Files">\n')
for file in FortranFiles:
    new.write(6*' '+'<sr file="%s" option="file"/>\n' % file)
new.write('</child>\n')
#include Files
new.write('<!-- Include files -->\n')
new.write('<parent name="Include Files">\n')
for file in IncludeFiles:
    new.write(6*' '+'<sr file="%s" option="file"/>\n' % file)
new.write('</parent>\n')
#Subroutines
new.write('<!-- Subroutines -->\n')
new.write('<table name="Routines and Functions" option="table">\n')
i = 0
list = Dict_Sub.keys()
list.sort(lambda x, y: cmp(string.lower(x), string.lower(y)))
for sub in list:
    if i == 0:
        new.write(3*' '+'<table option="line">\n')
    new.write(6*' '+'<table option="item">\n')
    new.write(9*' '+'<link name="%s" file="%s">%s</link>\n' % (sub,Dict_Sub[sub],sub))
    new.write(6*' '+'</table>\n')
    i = i+1
    if i == 3:
        i = 0
        new.write(3*' '+'</table>\n')
if i != 0:
    new.write('</table>\n')
new.write('</table>\n')
#Pointer variables
new.write('<!-- Pointer variables -->\n')
new.write('<table name="Pointer variables" option="table">\n')
i = 0
list = Pointer.keys()
list.sort(lambda x, y: cmp(string.lower(x), string.lower(y)))
for pointer in list:
    if i == 0:
        new.write(3*' '+'<table option="line">\n')
    new.write(6*' '+'<table option="item">\n')
    new.write(9*' '+'<link file="%s">%s</link>\n' % (Pointer[pointer],pointer))
    new.write(6*' '+'</table>\n')
    i = i+1
    if i == 6:
        i = 0
        new.write(3*' '+'</table>\n')
if i != 0:
    new.write('</table>\n')
new.write('</table>\n')
#Close
new.write('</module>\n')
new.close()
#Use xslt to build index.html 
if do_html:
    doc = libxml2.parseFile("%s/index.xml" % NEW)
    result = style.applyStylesheet(doc, None)
    style.saveResultToFilename("%s/index.html" % NEW, result, 0)

print "Stat: %d include files, %d fortran files converted." \
      % ( ninclude,nfortran)

print "Exit."
